package se.su.dsv.scipro.crosscutting;

import com.google.common.eventbus.EventBus;
import com.google.common.eventbus.Subscribe;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.project.ReviewerAssignedEvent;
import se.su.dsv.scipro.reviewing.*;

import javax.inject.Inject;
import javax.inject.Singleton;
import java.util.Date;
import java.util.function.Consumer;

@Singleton
class ReviewerAssignedDeadline {
    private final RoughDraftApprovalService roughDraftApprovalService;
    private final FinalSeminarApprovalService finalSeminarApprovalService;
    private final ReviewerDeadlineSettingsService reviewerDeadlineSettingsService;
    private final DaysService daysService;

    ReviewerAssignedDeadline(
            RoughDraftApprovalService roughDraftApprovalService,
            FinalSeminarApprovalService finalSeminarApprovalService,
            ReviewerDeadlineSettingsService reviewerDeadlineSettingsService,
            DaysService daysService)
    {
        this.roughDraftApprovalService = roughDraftApprovalService;
        this.finalSeminarApprovalService = finalSeminarApprovalService;
        this.reviewerDeadlineSettingsService = reviewerDeadlineSettingsService;
        this.daysService = daysService;
    }

    @Inject
    public ReviewerAssignedDeadline(
            RoughDraftApprovalService roughDraftApprovalService,
            FinalSeminarApprovalService finalSeminarApprovalService,
            ReviewerDeadlineSettingsService reviewerDeadlineSettingsService,
            DaysService daysService,
            EventBus eventBus)
    {
        this(roughDraftApprovalService, finalSeminarApprovalService, reviewerDeadlineSettingsService, daysService);
        eventBus.register(this);
    }

    @Subscribe
    public void reviewerAssigned(ReviewerAssignedEvent event) {
        ReviewerDeadlineSettings deadlineSettings = reviewerDeadlineSettingsService.getInstance();

        roughDraftApprovalService.findBy(event.getProject())
                .filter(rda -> !rda.isDecided())
                .foreach(setDeadline(getDeadline(deadlineSettings.getRoughDraftApproval())));

        finalSeminarApprovalService.findBy(event.getProject())
                .filter(fsa -> !fsa.isDecided())
                .foreach(setDeadline(getDeadline(deadlineSettings.getFinalSeminarApproval())));
    }

    protected Date getDeadline(int days) {
        return daysService.workDaysAfter(new Date(), days);
    }

    private Consumer<ReviewerApproval> setDeadline(Date date) {
        return approval -> approval.getCurrentDecision().setDeadline(date);
    }
}
