package se.su.dsv.scipro.file;

import com.google.inject.persist.Transactional;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;

public class FileDescriptionServiceImpl extends AbstractServiceImpl<FileDescription, Long> implements FileDescriptionService {

    public static final String FILE_ROOT = "/scipro-files";
    public static final int FILES_PER_SUBDIRECTORY = 1000;

    @Inject
    public FileDescriptionServiceImpl(Provider<EntityManager> em) {
        super(em, FileDescription.class, QFileDescription.fileDescription);
    }

    @Override
    @Transactional
    public FileDescription storeFile(FileUpload fileUpload) {
        return storeFile(fileUpload, new FileDescription());
    }

    private <T extends FileDescription> T storeFile(FileUpload fileUpload, T projectFile) {
        projectFile.setSize(fileUpload.getSize());
        projectFile.setMimeType(fileUpload.getContentType());
        projectFile.setName(fileUpload.getFileName());
        projectFile.setUploader(fileUpload.getUploader());
        T metaData = save(projectFile);
        storeData(fileUpload, metaData);
        return metaData;
    }

    protected <T extends FileDescription> void storeData(final FileUpload fileUpload, final T metaData) {
        fileUpload.handleData(input -> {
            metaData.storeActualData(input);
            return metaData;
        });
    }

    @Override
    public InputStream getData(FileDescription attachment) throws IOException {
        return Files.newInputStream(getPathFor(attachment));
    }

    private Path getPathFor(FileDescription fileDescription) {
        return FileSystems.getDefault().getPath(FILE_ROOT, getSubdirectory(fileDescription), Long.toString(fileDescription.getId()));
    }

    private String getSubdirectory(FileDescription fileDescription) {
        return Long.toString((fileDescription.getId() / FILES_PER_SUBDIRECTORY) * FILES_PER_SUBDIRECTORY + FILES_PER_SUBDIRECTORY);
    }

}
