package se.su.dsv.scipro.file;

import com.google.inject.persist.Transactional;
import com.querydsl.core.types.Predicate;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.project.Project;

import javax.inject.Inject;
import java.util.Collection;
import java.util.List;

public class ProjectFileServiceImpl implements ProjectFileService {
    private final FileDescriptionService fileDescriptionService;
    private final ProjectFileRepository projectFileRepository;

    @Inject
    ProjectFileServiceImpl(
            final FileDescriptionService fileDescriptionService,
            final ProjectFileRepository projectFileRepository)
    {
        this.fileDescriptionService = fileDescriptionService;
        this.projectFileRepository = projectFileRepository;
    }


    @Override
    @Transactional
    public ProjectFile store(final ProjectFileUpload projectFileUpload, final FileSource fileSource) {
        final FileDescription fileDescription = fileDescriptionService.storeFile(projectFileUpload);
        return promote(fileDescription, projectFileUpload.getProject(), fileSource);
    }

    @Override
    @Transactional
    public ProjectFile promote(final FileDescription fileDescription, final Project project, final FileSource fileSource) {
        final ProjectFile projectFile = new ProjectFile();
        projectFile.setFileDescription(fileDescription);
        projectFile.setFileSource(fileSource);
        projectFile.setProject(project);
        return projectFileRepository.save(projectFile);
    }

    @Override
    public List<ProjectFile> latestUpload(final Project project, final int amount) {
        return projectFileRepository.latestUpload(project, amount);
    }

    @Override
    public Collection<ProjectFile> getProjectFiles(final Project project, final Pageable pageable) {
        return projectFileRepository.getProjectFiles(project, pageable);
    }

    @Override
    public long countProjectFiles(final Project project) {
        return projectFileRepository.countProjectFiles(project);
    }

    @Override
    public <S extends ProjectFile> S save(final S entity) {
        return projectFileRepository.save(entity);
    }

    @Override
    public ProjectFile findOne(final Long id) {
        return projectFileRepository.findOne(id);
    }

    @Override
    public void delete(final Long id) {
        projectFileRepository.delete(id);
    }

    @Override
    public void delete(final ProjectFile entity) {
        projectFileRepository.delete(entity);
    }

    @Override
    public long count() {
        return projectFileRepository.count();
    }

    @Override
    public List<ProjectFile> findAll() {
        return projectFileRepository.findAll();
    }

    @Override
    public List<ProjectFile> findAll(final Pageable pageable) {
        return projectFileRepository.findAll(QProjectFile.projectFile.isNotNull(), pageable);
    }

    @Override
    public long count(final Predicate predicate) {
        return projectFileRepository.count(predicate);
    }

    @Override
    public ProjectFile findOne(final Predicate predicate) {
        return projectFileRepository.findOne(predicate);
    }

    @Override
    public List<ProjectFile> findAll(final Predicate predicate) {
        return projectFileRepository.findAll(predicate);
    }

    @Override
    public List<ProjectFile> findAll(final Predicate predicate, final Pageable pageable) {
        return projectFileRepository.findAll(predicate, pageable);
    }
}
