package se.su.dsv.scipro.finalthesis;

import com.querydsl.core.types.dsl.BooleanExpression;
import com.querydsl.jpa.impl.JPAQuery;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.project.ProjectStatus;
import se.su.dsv.scipro.report.QReviewerGradingReport;
import se.su.dsv.scipro.system.GenericRepo;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.List;

import static com.querydsl.core.types.dsl.Expressions.allOf;

public class FinalThesisRepoImpl extends GenericRepo<FinalThesis, Long> implements FinalThesisRepo {
    @Inject
    public FinalThesisRepoImpl(Provider<EntityManager> em) {
        super(em, FinalThesis.class, QFinalThesis.finalThesis);
    }

    @Override
    public List<FinalThesis> reviewers(final User reviewer, final boolean showAll, final Pageable pageable) {
        return findAll(getFilter(reviewer, showAll), pageable);
    }

    @Override
    public Long countReviewers(User reviewer, final boolean showAll) {
        return count(getFilter(reviewer, showAll));
    }

    private BooleanExpression getFilter(final User reviewer, final boolean showAll) {
        return showAll ? allFilter(reviewer) : unfinishedFilter(reviewer);
    }

    private BooleanExpression allFilter(final User reviewer) {
        return allOf(QFinalThesis.finalThesis.status.eq(FinalThesis.Status.APPROVED),
                QFinalThesis.finalThesis.project.reviewers.any().eq(reviewer));
    }

    private BooleanExpression unfinishedFilter(final User reviewer) {
        return allOf(QFinalThesis.finalThesis.status.eq(FinalThesis.Status.APPROVED),
                QFinalThesis.finalThesis.project.projectStatus.eq(ProjectStatus.ACTIVE),
                QFinalThesis.finalThesis.project.reviewers.any().eq(reviewer),
                QFinalThesis.finalThesis.project.notIn(new JPAQuery<Project>()
                        .select(QReviewerGradingReport.reviewerGradingReport.project)
                        .from(QReviewerGradingReport.reviewerGradingReport)
                        .where(allOf(
                                QReviewerGradingReport.reviewerGradingReport.submitted.isTrue(),
                                QReviewerGradingReport.reviewerGradingReport.project.reviewers.any().eq(reviewer)))
                        ));
    }
}