package se.su.dsv.scipro.firstmeeting;

import se.su.dsv.scipro.activityplan.Activity;
import se.su.dsv.scipro.activityplan.ActivityPlanFacade;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Date;
import java.util.Optional;

public class FirstMeetingServiceImpl extends AbstractServiceImpl<ProjectFirstMeeting, Long> implements FirstMeetingService {
    private final ActivityPlanFacade activityPlanFacade;

    @Inject
    public FirstMeetingServiceImpl(final Provider<EntityManager> em, final ActivityPlanFacade activityPlanFacade) {
        super(em, ProjectFirstMeeting.class, QProjectFirstMeeting.projectFirstMeeting);
        this.activityPlanFacade = activityPlanFacade;
    }

    @Override
    public Optional<ProjectFirstMeeting> findByProject(final Project project) {
        return Optional.ofNullable(findOne(QProjectFirstMeeting.projectFirstMeeting.activity.activityPlan.project.eq(project)));
    }

    @Override
    public ProjectFirstMeeting schedule(final Project project, final Date date, final String room, final String description) {
        final Optional<ProjectFirstMeeting> optFirstMeeting = findByProject(project);
        final ProjectFirstMeeting firstMeeting = optFirstMeeting
                .orElseGet(ProjectFirstMeeting::new);
        final Activity activity = optFirstMeeting
                .map(ProjectFirstMeeting::getActivity)
                .orElseGet(() -> createNewActivity(project));

        final Activity updated = updateActivity(activity, date, description);
        return updateFirstMeeting(firstMeeting, room, updated);
    }

    private Activity createNewActivity(final Project project) {
        final Activity activity = new Activity();
        activity.setActivityPlan(activityPlanFacade.retrieveActivityPlan(project));
        activity.setTitle("First meeting");
        return activity;
    }

    private Activity updateActivity(final Activity activity, final Date date, final String description) {
        activity.setDate(date);
        activity.setDescription(description);
        return activityPlanFacade.saveActivity(activity);
    }

    private ProjectFirstMeeting updateFirstMeeting(final ProjectFirstMeeting created, final String room, final Activity createdActivity) {
        created.setActivity(createdActivity);
        created.setRoom(room);
        return save(created);
    }
}
