package se.su.dsv.scipro.oauth;

import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;

import javax.inject.Inject;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.core.Form;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.util.Set;
import java.util.stream.Collectors;

class OAuthServiceImpl implements OAuthService {
    private final OAuthSettingsRepository oAuthSettingsRepository;

    @Inject
    OAuthServiceImpl(final OAuthSettingsRepository oAuthSettingsRepository) {
        this.oAuthSettingsRepository = oAuthSettingsRepository;
    }

    @Override
    public String authorizeUrl(final String state, final Set<String> scopes) {
        final OAuthSettings settings = oAuthSettingsRepository.getSettings();
        final StringBuilder authorizationUri = new StringBuilder()
                .append(settings.getUri())
                .append("/authorize")
                .append("?response_type=").append("code")
                .append("&client_id=").append(settings.getClientId())
                .append("&redirect_uri=").append(settings.getRedirectUri())
                .append("&scope=").append(scopes.stream().collect(Collectors.joining(" ")));
        if (state != null) {
            authorizationUri.append("&state=").append(state);
        }
        return authorizationUri.toString();
    }

    @Override
    public TokenResponse exchangeForToken(final String code) {
        final OAuthSettings settings = oAuthSettingsRepository.getSettings();

        final Form form = new Form();
        form.param("grant_type", "authorization_code");
        form.param("code", code);
        form.param("redirect_uri", settings.getRedirectUri());

        final Client client = ClientBuilder.newClient();
        final Response response = client.target(settings.getUri())
                .register(HttpAuthenticationFeature.basic(settings.getClientId(), settings.getClientSecret()))
                .path("exchange")
                .request(MediaType.APPLICATION_JSON_TYPE)
                .post(Entity.form(form));

        if (response.getStatus() == Response.Status.OK.getStatusCode()) {
            return response.readEntity(TokenResponse.class);
        }
        else {
            return null;
        }
    }
}
