package se.su.dsv.scipro.oauth;

import com.google.inject.persist.Transactional;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

class OAuthSettingsRepositoryImpl implements OAuthSettingsRepository {

    private final Provider<EntityManager> em;

    @Inject
    OAuthSettingsRepositoryImpl(final Provider<EntityManager> em) {
        this.em = em;
    }

    @Override
    @Transactional
    public OAuthSettings getSettings() {
        final EntityManager entityManager = em.get();
        final OAuthSettings settings = entityManager.find(OAuthSettings.class, OAuthSettings.ID);
        if (settings == null) {
            final OAuthSettings defaultSettings = new OAuthSettings();
            entityManager.persist(defaultSettings);
            return defaultSettings;
        }
        else {
            return settings;
        }
    }

    @Override
    @Transactional
    public void save(final OAuthSettings settings) {
        em.get().merge(settings);
    }
}
