package se.su.dsv.scipro.plagiarism;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.plagiarism.urkund.UrkundService;
import se.su.dsv.scipro.plagiarism.urkund.UrkundSettings;
import se.su.dsv.scipro.workerthreads.AbstractWorker;

import javax.inject.Inject;
import javax.inject.Provider;
import java.util.List;

public class PlagiarismSubmitter extends AbstractWorker {
    private static final Logger LOG = LoggerFactory.getLogger(PlagiarismSubmitter.class);

    private final Provider<UrkundSettings> urkundSettingsProvider;
    private final PlagiarismRequestRepository plagiarismRequestRepository;
    private final UrkundService urkundService;

    @Inject
    PlagiarismSubmitter(
            final Provider<UrkundSettings> urkundSettingsProvider,
            final PlagiarismRequestRepository plagiarismRequestRepository,
            final UrkundService urkundService)
    {
        this.urkundSettingsProvider = urkundSettingsProvider;
        this.plagiarismRequestRepository = plagiarismRequestRepository;
        this.urkundService = urkundService;
    }

    @Override
    protected void doWork() {
        if (!urkundSettingsProvider.get().isEnabled()) {
            return;
        }
        List<PlagiarismRequest> requests = plagiarismRequestRepository.findAll();
        for (PlagiarismRequest request : requests) {
            urkundService.submit(request.getFileDescription(), request.getReceiver());
            plagiarismRequestRepository.delete(request);
            LOG.info("Submitted {} to URKUND", request);
        }
    }
}
