package se.su.dsv.scipro.plagiarism.urkund;

import lombok.*;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;
import java.time.Instant;

@Data
@Entity
@Table(name = "urkund_submission")
@EqualsAndHashCode(callSuper = true)
@Setter(AccessLevel.PACKAGE)
public class UrkundSubmission extends DomainObject {

    public enum State {
        SUBMISSION_FAILED, SUBMITTED, REJECTED, ACCEPTED, ANALYZED, ERROR;

        public boolean isFinal() {
            return !(this == SUBMITTED || this == ACCEPTED);
        }
    }

    @Id
    @GeneratedValue
    private Long id;

    @ManyToOne
    @Getter(AccessLevel.PACKAGE)
    private User receiver;

    @Basic
    private String analysisAddress;

    @Column(nullable = false)
    @Enumerated(EnumType.STRING)
    private State state;

    @Column(nullable = false)
    private Instant submitted;

    @Column(nullable = false)
    @Getter(AccessLevel.PACKAGE)
    private Instant nextPoll;

    @Column(nullable = false)
    @Enumerated(EnumType.STRING)
    @Getter(AccessLevel.PACKAGE)
    private PollingDelay pollingDelay = PollingDelay.FIRST;

    @OneToOne(optional = false)
    private FileDescription fileDescription;

    @Basic(optional = false)
    private String message;

    @Basic
    private String reportUrl;

    @Basic
    private Float significance;

}
