package se.su.dsv.scipro.plagiarism.urkund;

import com.querydsl.jpa.impl.JPAQuery;
import se.su.dsv.scipro.file.FileDescription;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Collection;
import java.util.Optional;

class UrkundSubmissionRepositoryImpl implements UrkundSubmissionRepository {
    private final Provider<EntityManager> entityManagerProvider;

    @Inject
    UrkundSubmissionRepositoryImpl(final Provider<EntityManager> entityManagerProvider) {
        this.entityManagerProvider = entityManagerProvider;
    }

    @Override
    public Optional<UrkundSubmission> findByFile(final FileDescription file) {
        final UrkundSubmission urkundSubmission = new JPAQuery<UrkundSubmission>(entityManagerProvider.get())
                .from(QUrkundSubmission.urkundSubmission)
                .where(QUrkundSubmission.urkundSubmission.fileDescription.eq(file))
                .fetchOne();
        return Optional.ofNullable(urkundSubmission);
    }

    @Override
    public UrkundSubmission save(final UrkundSubmission submission) {
        final EntityManager em = entityManagerProvider.get();
        if (em.contains(submission)) {
            return em.merge(submission);
        }
        else {
            em.persist(submission);
            return submission;
        }
    }

    @Override
    public Collection<UrkundSubmission> findNotDone() {
        return new JPAQuery<UrkundSubmission>(entityManagerProvider.get())
                .from(QUrkundSubmission.urkundSubmission)
                .where(QUrkundSubmission.urkundSubmission.state.in(
                        UrkundSubmission.State.SUBMITTED,
                        UrkundSubmission.State.ACCEPTED))
                .fetch();
    }

    @Override
    public void delete(final UrkundSubmission urkundSubmission) {
        entityManagerProvider.get().remove(urkundSubmission);
    }
}
