package se.su.dsv.scipro.projectpartner;

import com.querydsl.core.types.dsl.Expressions;
import com.querydsl.jpa.JPAExpressions;
import org.joda.time.DateTime;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.match.QIdea;
import se.su.dsv.scipro.match.QIdeaParticipation;
import se.su.dsv.scipro.system.AbstractServiceImpl;
import se.su.dsv.scipro.system.ProjectType;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.Date;
import java.util.List;

public class ProjectPartnerServiceImpl extends AbstractServiceImpl<ProjectPartner,Long> implements ProjectPartnerService {

    @Inject
    public ProjectPartnerServiceImpl(Provider<EntityManager> em) {
        super(em, ProjectPartner.class, QProjectPartner.projectPartner);
    }

    @Override
    public List<ProjectPartner> getProjectPartnerInSpan(int days, final ApplicationPeriod applicationPeriod, final ProjectType projectType, Pageable pageable) {
        if (days == 0) {
            return findAll(QProjectPartner.projectPartner.projectType.eq(projectType), pageable);
        }
        Date date = new DateTime().minusDays(days).toDate();
        return findAll(QProjectPartner.projectPartner.dateCreated.after(date).and(QProjectPartner.projectPartner.projectType.eq(projectType)).and(QProjectPartner.projectPartner.applicationPeriod.eq(applicationPeriod)), pageable);
    }

    @Override
    public long countProjectPartnerInSpan(int days, final ApplicationPeriod applicationPeriod, final ProjectType projectType) {
        if (days == 0) {
            return count(QProjectPartner.projectPartner.projectType.eq(projectType));
        }
        Date date = new DateTime().minusDays(days).toDate();
        return count(QProjectPartner.projectPartner.dateCreated.after(date).and(QProjectPartner.projectPartner.projectType.eq(projectType)).and(QProjectPartner.projectPartner.applicationPeriod.eq(applicationPeriod)));
    }

    @Override
    public List<ProjectPartner> getPartnersWithoutIdea(final ApplicationPeriod applicationPeriod) {
        return findAll(Expressions.allOf(
                QProjectPartner.projectPartner.user.notIn(
                        JPAExpressions.select(QIdeaParticipation.ideaParticipation.user)
                                .from(QIdea.idea)
                                .join(QIdea.idea.ideaParticipations, QIdeaParticipation.ideaParticipation)
                                .where(QIdea.idea.applicationPeriod.eq(applicationPeriod))),
                QProjectPartner.projectPartner.applicationPeriod.eq(applicationPeriod)
        ));
    }
}
