package se.su.dsv.scipro.report.calculators.vt17;

import se.su.dsv.scipro.report.GradeCalculator;
import se.su.dsv.scipro.report.GradingCriterion;
import se.su.dsv.scipro.report.GradingReport;

public abstract class AbstractBachelorGradeCalculator implements GradeCalculator {

    private static final int NUMBER_OF_PROJECT_CRITERIA_NEEDED_TO_GET_FX = 8;
    private static final int INDIVIDUAL_CRITERIA_POINTS_NEEDED_TO_GET_FX = 2;
    private static final int POINTS_NEEDED_FOR_A = 26;
    private static final int POINTS_NEEDED_FOR_B = 24;
    private static final int POINTS_NEEDED_FOR_C = 20;
    private static final int POINTS_NEEDED_FOR_D = 18;

    @Override
    public GradingReport.Grade getGrade(GradingReport gradingReport) {
        return new BachelorCalculation(getGradingCriteria(gradingReport)).getGrade();
    }

    @Override
    public long getPoints(GradingReport gradingReport) {
        return new BachelorCalculation(getGradingCriteria(gradingReport)).calculateTotalPoints();
    }

    public abstract Iterable<? extends GradingCriterion> getGradingCriteria(GradingReport gradingReport);

    private static class BachelorCalculation extends Calculation {
        private final Iterable<? extends GradingCriterion> gradingCriteria;

        public BachelorCalculation(Iterable<? extends GradingCriterion> gradingCriteria) {
            this.gradingCriteria = gradingCriteria;
        }

        @Override
        protected Iterable<? extends GradingCriterion> getGradingCriteria() {
            return gradingCriteria;
        }

        protected GradingReport.Grade convertPointsToGrade() {
            int total = calculateTotalPoints();
            if (total >= POINTS_NEEDED_FOR_A && hasPointsForDeadlineCriterion()) {
                return GradingReport.Grade.A;
            } else if (total >= POINTS_NEEDED_FOR_B) {
                return GradingReport.Grade.B;
            } else if (total >= POINTS_NEEDED_FOR_C) {
                return GradingReport.Grade.C;
            } else if (total >= POINTS_NEEDED_FOR_D) {
                return GradingReport.Grade.D;
            } else {
                return GradingReport.Grade.E;
            }
        }

        private boolean hasPointsForDeadlineCriterion() {
            for (GradingCriterion gradingCriterion : gradingCriteria) {
                if (gradingCriterion.getTitle().equals("Ö4 Deadlines") && gradingCriterion.getPoints() == 1) {
                    return true;
                }
            }
            return false;
        }

        @Override
        protected int getNumberOfProjectCriteriaRequiredToGetFx() {
            return NUMBER_OF_PROJECT_CRITERIA_NEEDED_TO_GET_FX;
        }

        @Override
        protected int getIndividualCriteriaPointsRequiredToGetFx() {
            return INDIVIDUAL_CRITERIA_POINTS_NEEDED_TO_GET_FX;
        }

    }
}
