package se.su.dsv.scipro.sukat;

import lombok.experimental.Delegate;
import se.su.dsv.scipro.system.Username;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.*;
import java.util.Hashtable;
import java.util.Optional;

public class LDAP implements Sukat {
    @Override
    public Optional<String> email(final Username username) {
        try (CloseableDirContext context = createContext()) {
            final SearchControls searchControls = new SearchControls();
            searchControls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            final NamingEnumeration<SearchResult> results = context.search("dc=su,dc=se", "uid=" + getUid(username), searchControls);
            while (results.hasMore()) {
                final SearchResult result = results.next();
                final Attribute mail = result.getAttributes().get("mail");
                if (mail != null) {
                    return Optional.of((String) mail.get());
                }
            }
            return Optional.empty();
        } catch (NamingException e) {
            throw new RuntimeException(e);
        }
    }

    private String getUid(final Username username) {
        final int i = username.getUsername().indexOf('@');
        return username.getUsername().substring(0, i);
    }

    private CloseableDirContext createContext() throws NamingException {
        // Set up the environment for creating the initial context
        Hashtable<String, String> env = new Hashtable<>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, "ldap://ldap.su.se:389/");
        return new CloseableDirContext(new InitialDirContext(env));
    }

    private static class CloseableDirContext implements DirContext, AutoCloseable {
        @Delegate
        private final DirContext wrapped;

        private CloseableDirContext(final DirContext wrapped) {
            this.wrapped = wrapped;
        }
    }
}
