package se.su.dsv.scipro.system;

import com.querydsl.jpa.JPAExpressions;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.security.auth.roles.Roles;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;
import java.util.*;

import static com.querydsl.core.types.dsl.Expressions.allOf;
import static com.querydsl.core.types.dsl.Expressions.anyOf;

public class UserRepoImpl extends GenericRepo<User, Long> implements UserRepo {
    private final Set<UserSearchService> userSearchServices;

    @Inject
    public UserRepoImpl(Provider<EntityManager> em, Set<UserSearchService> userSearchServices) {
        super(em, User.class, QUser.user);
        this.userSearchServices = userSearchServices;
    }

    @Override
    public Collection<User> findByNameIncludes(String term, Roles role, Pageable pageable) {
        QUser user = QUser.user;
        final Set<User> results = new HashSet<>();
        results.addAll(findAll(allOf(
                user.deleted.isFalse(),
                user.roles.any().eq(role),
                anyOf(
                        user.firstName.concat(" ").concat(user.lastName).contains(term),
                        user.emailAddress.contains(term),
                        user.in(JPAExpressions
                                .select(QUsername.username1.user)
                                .from(QUsername.username1)
                                .where(QUsername.username1.username.contains(term)))
                )
        ), pageable));

        long remaining = pageable.getLimit() - results.size();
        for (UserSearchService userSearchService : userSearchServices) {
            final Collection<? extends User> users = remaining > 0
                    ? userSearchService.search(term, role, remaining)
                    : Collections.<User>emptySet();

            results.addAll(users);
            remaining -= users.size();
        }

        return results;
    }

}
