package se.su.dsv.scipro.finalseminar;

import java.time.Instant;
import java.util.Date;
import java.util.function.Function;

public abstract class OppositionRegistrationStatus {
    OppositionRegistrationStatus() { }

    public abstract <A> A fold(
            Function<UngradedOpposition, A> a,
            Function<FinalSeminarCancelled, A> b,
            Function<AlreadyParticipating, A> c,
            Function<AlreadyHappened, A> d,
            Function<TooManyOpponents, A> e,
            Function<PriorityForSeminarAuthors, A> f,
            Function<AlreadyOpposed, A> g);
}

final class UngradedOpposition extends OppositionRegistrationStatus {
    private final FinalSeminarOpposition ungradedOpposition;

    UngradedOpposition(final FinalSeminarOpposition ungradedOpposition) {
        this.ungradedOpposition = ungradedOpposition;
    }

    public FinalSeminarOpposition getUngradedOpposition() {
        return ungradedOpposition;
    }

    @Override
    public <A> A fold(
            final Function<UngradedOpposition, A> a,
            final Function<FinalSeminarCancelled, A> b,
            final Function<AlreadyParticipating, A> c,
            final Function<AlreadyHappened, A> d,
            final Function<TooManyOpponents, A> e,
            final Function<PriorityForSeminarAuthors, A> f,
            final Function<AlreadyOpposed, A> g)
    {
        return a.apply(this);
    }
}

final class AlreadyOpposed extends OppositionRegistrationStatus {
    private final FinalSeminarOpposition opposed;

    AlreadyOpposed(final FinalSeminarOpposition opposed) {
        this.opposed = opposed;
    }

    public FinalSeminarOpposition getOpposed() {
        return opposed;
    }

    @Override
    public <A> A fold(
            final Function<UngradedOpposition, A> a,
            final Function<FinalSeminarCancelled, A> b,
            final Function<AlreadyParticipating, A> c,
            final Function<AlreadyHappened, A> d,
            final Function<TooManyOpponents, A> e,
            final Function<PriorityForSeminarAuthors, A> f,
            final Function<AlreadyOpposed, A> g)
    {
        return g.apply(this);
    }
}

final class FinalSeminarCancelled extends OppositionRegistrationStatus {
    @Override
    public <A> A fold(
            final Function<UngradedOpposition, A> a,
            final Function<FinalSeminarCancelled, A> b,
            final Function<AlreadyParticipating, A> c,
            final Function<AlreadyHappened, A> d,
            final Function<TooManyOpponents, A> e,
            final Function<PriorityForSeminarAuthors, A> f,
            final Function<AlreadyOpposed, A> g)
    {
        return b.apply(this);
    }
}

final class AlreadyParticipating extends OppositionRegistrationStatus {
    @Override
    public <A> A fold(
            final Function<UngradedOpposition, A> a,
            final Function<FinalSeminarCancelled, A> b,
            final Function<AlreadyParticipating, A> c,
            final Function<AlreadyHappened, A> d,
            final Function<TooManyOpponents, A> e,
            final Function<PriorityForSeminarAuthors, A> f,
            final Function<AlreadyOpposed, A> g)
    {
        return c.apply(this);
    }
}

final class AlreadyHappened extends OppositionRegistrationStatus {
    private final Date date;

    AlreadyHappened(final Date date) {
        this.date = date;
    }

    public Date getDate() {
        return date;
    }

    @Override
    public <A> A fold(
            final Function<UngradedOpposition, A> a,
            final Function<FinalSeminarCancelled, A> b,
            final Function<AlreadyParticipating, A> c,
            final Function<AlreadyHappened, A> d,
            final Function<TooManyOpponents, A> e,
            final Function<PriorityForSeminarAuthors, A> f,
            final Function<AlreadyOpposed, A> g)
    {
        return d.apply(this);
    }
}

final class TooManyOpponents extends OppositionRegistrationStatus {
    private final int registeredOpponents;
    private final int maxOpponents;

    TooManyOpponents(final int registeredOpponents, final int maxOpponents) {
        this.registeredOpponents = registeredOpponents;
        this.maxOpponents = maxOpponents;
    }

    public int getRegisteredOpponents() {
        return registeredOpponents;
    }

    public int getMaxOpponents() {
        return maxOpponents;
    }

    @Override
    public <A> A fold(
            final Function<UngradedOpposition, A> a,
            final Function<FinalSeminarCancelled, A> b,
            final Function<AlreadyParticipating, A> c,
            final Function<AlreadyHappened, A> d,
            final Function<TooManyOpponents, A> e,
            final Function<PriorityForSeminarAuthors, A> f,
            final Function<AlreadyOpposed, A> g)
    {
        return e.apply(this);
    }
}

final class PriorityForSeminarAuthors extends OppositionRegistrationStatus {
    private final int priorityDays;
    private final Instant priorityEnd;

    PriorityForSeminarAuthors(final int priorityDays, final Instant priorityEnd) {
        this.priorityDays = priorityDays;
        this.priorityEnd = priorityEnd;
    }

    public int getPriorityDays() {
        return priorityDays;
    }

    public Instant getPriorityEnd() {
        return priorityEnd;
    }

    @Override
    public <A> A fold(
            final Function<UngradedOpposition, A> a,
            final Function<FinalSeminarCancelled, A> b,
            final Function<AlreadyParticipating, A> c,
            final Function<AlreadyHappened, A> d,
            final Function<TooManyOpponents, A> e,
            final Function<PriorityForSeminarAuthors, A> f,
            final Function<AlreadyOpposed, A> g)
    {
        return f.apply(this);
    }
}