package se.su.dsv.scipro.plagiarism.urkund;

import com.google.inject.AbstractModule;
import com.google.inject.Module;
import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeMatcher;
import org.junit.Test;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.sukat.Sukat;
import se.su.dsv.scipro.test.GuiceTest;

import javax.inject.Inject;
import java.time.Instant;
import java.util.Optional;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

public class UrkundSubmissionRepositoryTest extends GuiceTest {

    @Inject
    private UrkundSubmissionRepository submissionRepository;
    
    @Override
    protected Module moduleUnderTest() {
        return new AbstractModule() {
            @Override
            protected void configure() {
                install(new UrkundModule());
                bind(Sukat.class).toInstance((username) -> Optional.empty());
            }
        };
    }

    @Test
    public void save() throws Exception {
        final Instant submitted = Instant.now();
        final UrkundSubmission submission = new UrkundSubmission();
        submission.setState(UrkundSubmission.State.SUBMITTED);
        submission.setMessage("Hi");
        submission.setSubmitted(submitted);
        submission.setNextPoll(submitted);
        final FileDescription file = save(new FileDescription());
        submission.setFileDescription(file);

        final UrkundSubmission saved = submissionRepository.save(submission);

        final Optional<UrkundSubmission> byFile = submissionRepository.findByFile(file);

        assertThat(byFile, is(some(equalTo(saved))));
    }

    private <A> Matcher<Optional<A>> some(final Matcher<A> a) {
        return new TypeSafeMatcher<Optional<A>>() {
            @Override
            protected boolean matchesSafely(final Optional<A> item) {
                return item.map(a::matches).orElse(false);
            }

            @Override
            public void describeTo(final Description description) {
                description.appendText("<Optional.some(");
                a.describeTo(description);
                description.appendText(")>");
            }
        };
    }
}
