package se.su.dsv.scipro.test;

import org.hamcrest.Description;
import org.hamcrest.Matcher;
import org.hamcrest.TypeSafeDiagnosingMatcher;
import se.su.dsv.scipro.util.Either;

public final class Matchers {
    public static <E, A> Matcher<Either<E, A>> isRight(Matcher<? super A> subMatcher) {
        return new TypeSafeDiagnosingMatcher<Either<E, A>>() {
            @Override
            protected boolean matchesSafely(final Either<E, A> item, final Description mismatchDescription) {
                return item.fold(
                        e -> {
                            mismatchDescription.appendText("Left(").appendValue(e).appendText(")");
                            return false;
                        },
                        a -> {
                            if (!subMatcher.matches(a)) {
                                mismatchDescription.appendText("Right(");
                                subMatcher.describeMismatch(a, mismatchDescription);
                                mismatchDescription.appendText(")");
                                return false;
                            }
                            return true;
                        });
            }

            @Override
            public void describeTo(final Description description) {
                description.appendText("Right(").appendDescriptionOf(subMatcher).appendText(")");
            }
        };
    }

    public static <E, A> Matcher<Either<E, A>> isLeft(Matcher<? super E> subMatcher) {
        return new TypeSafeDiagnosingMatcher<Either<E, A>>() {
            @Override
            protected boolean matchesSafely(final Either<E, A> item, final Description mismatchDescription) {
                return item.fold(
                        e -> {
                            if (!subMatcher.matches(e)) {
                                mismatchDescription.appendText("Left(");
                                subMatcher.describeMismatch(e, mismatchDescription);
                                mismatchDescription.appendText(")");
                                return false;
                            }
                            return true;
                        },
                        a -> {
                            mismatchDescription.appendText("Right(").appendValue(a).appendText(")");
                            return false;
                        });
            }

            @Override
            public void describeTo(final Description description) {
                description.appendText("Left(").appendDescriptionOf(subMatcher).appendText(")");
            }
        };
    }

}
