package se.su.dsv.scipro.integration.daisy.workers;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import se.su.dsv.scipro.io.dto.STATUS;
import se.su.dsv.scipro.daisyExternal.http.DaisyAPI;
import se.su.dsv.scipro.project.*;
import se.su.dsv.scipro.util.Option;
import se.su.dsv.scipro.workerthreads.AbstractWorker;

import javax.inject.Inject;
import javax.inject.Named;

@Named
public class ProjectFinalizer extends AbstractWorker {

    private static final Logger LOG = LoggerFactory.getLogger(ProjectFinalizer.class);

    private final ProjectService projectService;
    private final DaisyAPI daisyAPI;

    @Inject
    public ProjectFinalizer(ProjectService projectService, DaisyAPI daisyAPI) {
        this.projectService = projectService;
        this.daisyAPI = daisyAPI;
    }

    @Override
    protected void doWork() {
        beginTransaction();
        Iterable<Project> projects = projectService.findAll(QProject.project.projectStatus.eq(ProjectStatus.ACTIVE));
        for (Project project : projects) {
            try {
                Option.of(project.getIdentifier())
                        .flatMap(daisyAPI::getThesis)
                        .filter(thesis -> thesis.getStatus().equals(STATUS.FINISHED))
                        .foreach(thesis -> {
                            projectService.complete(project);
                            LOG.info("Finalized project ({})", project);
                        });
            } catch (Exception e) {
                LOG.info("Failed to finalize project ({})", project, e);
            }
        }
        commitTransaction();
    }
}
