package se.su.dsv.scipro.admin.pages.settings;

import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import se.su.dsv.scipro.admin.pages.AbstractAdminSystemPage;
import se.su.dsv.scipro.oauth.OAuthSettings;
import se.su.dsv.scipro.oauth.OAuthSettingsRepository;

import javax.inject.Inject;

public class AdminOAuthSettingsPage extends AbstractAdminSystemPage {
    @Inject
    private OAuthSettingsRepository oAuthSettingsRepository;

    public AdminOAuthSettingsPage() {
        add(new OAuthSettingsForm("form", getSettings()));
    }

    private IModel<OAuthSettings> getSettings() {
        return new LoadableDetachableModel<OAuthSettings>() {
            @Override
            protected OAuthSettings load() {
                return oAuthSettingsRepository.getSettings();
            }
        };
    }

    private class OAuthSettingsForm extends Form<OAuthSettings> {
        public OAuthSettingsForm(final String id, final IModel<OAuthSettings> settings) {
            super(id, new CompoundPropertyModel<>(settings));

            add(new TextField<>("uri", String.class));
            add(new TextField<>("redirectUri", String.class));
            add(new TextField<>("clientId", String.class));
            add(new TextField<>("clientSecret", String.class));
        }

        @Override
        protected void onSubmit() {
            oAuthSettingsRepository.save(getModelObject());
        }
    }
}
