package se.su.dsv.scipro.components;

import org.apache.wicket.model.IModel;
import org.wicketstuff.select2.ChoiceProvider;
import org.wicketstuff.select2.Response;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class UserAutoComplete extends AutoCompleteChoice<User> {

    @Inject
    private UserService userService;

    public UserAutoComplete(String id) {
        super(id);
    }

    public UserAutoComplete(String id, IModel<User> userModel) {
        super(id);
        setModel(userModel);
        setType(User.class);
    }

    @Override
    protected ChoiceProvider<User> getChoiceProvider() {
        return new UserProvider();
    }

    private class UserProvider extends ChoiceProvider<User> {

        @Override
        public String getDisplayValue(User user) {
            StringBuilder sb = new StringBuilder();
            sb.append(user.getFullName());
            if (user.getEmailAddress() != null) {
                sb.append(" <");
                sb.append(user.getEmailAddress());
                sb.append(">");
            }
            return sb.toString();
        }

        @Override
        public String getIdValue(User choice) {
            return String.valueOf(choice.getId());
        }

        @Override
        public void query(String term, int page, Response<User> response) {
            response.addAll(userService.getAutoCompleteUsers(term, MAX_RESULTS));
        }

        @Override
        public Collection<User> toChoices(Collection<String> ids) {
            List<User> users = new ArrayList<>();
            for (String id : ids) {
                users.add(userService.findOne(Long.valueOf(id)));
            }
            return users;
        }
    }
}
