package se.su.dsv.scipro.components.datatables;

import org.apache.wicket.Component;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.export.AbstractExportableColumn;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.components.DisplayMultiplesPanel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.system.User;

import java.util.List;
import java.util.stream.Collectors;

public abstract class MultipleUsersColumn<T, S> extends AbstractExportableColumn<T, S> {
    public MultipleUsersColumn(final IModel<String> displayModel) {
        super(displayModel);
    }

    @Override
    public void populateItem(final Item<ICellPopulator<T>> cellItem, final String componentId, final IModel<T> rowModel) {
        cellItem.add(new DisplayMultiplesPanel<User>(componentId, getUsers(rowModel)) {
            @Override
            public Component getComponent(final String componentId, final IModel<User> t) {
                return new UserLinkPanel(componentId, t);
            }
        });
    }

    @Override
    public IModel<?> getDataModel(final IModel<T> rowModel) {
        return new AbstractReadOnlyModel<String>() {
            @Override
            public String getObject() {
                return getUsers(rowModel)
                        .getObject()
                        .stream()
                        .map(User::getFullName)
                        .collect(Collectors.joining(", "));
            }
        };
    }

    protected abstract IModel<? extends List<User>> getUsers(final IModel<T> rowModel);
}
