package se.su.dsv.scipro.data;

import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.GenericService;

import java.util.Collection;
import java.util.Collections;
import java.util.stream.Collectors;

public final class DetachableServiceModelCollection<DO extends DomainObject> implements IModel<Collection<DO>> {

    private static final long serialVersionUID = -7932592734004059133L;

    private transient boolean attached = false;
    private transient Collection<DO> object = Collections.emptySet();

    private GenericService<DO, Long> service;
    private Collection<Long> ids = Collections.emptySet();

    public DetachableServiceModelCollection(GenericService<DO, Long> service) {
        this.service = service;
    }

    public DetachableServiceModelCollection(GenericService<DO, Long> service, Collection<DO> object) {
        this(service);

        setObject(object);
    }

    @Override
    public Collection<DO> getObject() {
        if (!attached) {
            reload();
        }
        return object;
    }

    @Override
    public void setObject(Collection<DO> object) {
        attached = true;
        ids = object.stream()
                .map(DomainObject::getId)
                .collect(Collectors.toSet());
        this.object = object;
    }

    @Override
    public void detach() {
        // Workaround for Wicket only calling setObject if they are not
        // equal but since Hibernate injects id into the same reference
        // they are always equal so setObject will never get called.
        if (attached) {
            ids = object.stream()
                    .map(DomainObject::getId)
                    .collect(Collectors.toSet());
        }
        attached = false;
        object = null;
    }

    private void reload() {
        final Collection<DO> objects = ids.stream()
                .map(service::findOne)
                .collect(Collectors.toSet());
        setObject(objects);
    }
}
