package se.su.dsv.scipro.data;

import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;

import java.util.ArrayList;
import java.util.List;

/**
 * Model to use in a ListView (or another repeater component) when you want to hide some list items conditionally.
 */
public abstract class FilteredListModel<T> extends LoadableDetachableModel<List<T>>{

	private IModel<List<T>> inner;

    @Override
    protected void onDetach(){
        inner.detach();
    }
    
	public FilteredListModel(IModel<List<T>> inner){
        this.inner = inner;
    }

    @Override
    protected final List<T> load(){
        List<? extends T> input = inner.getObject();
        List<T> result = new ArrayList<>(input.size());
        for(T t : input){
            if(accept(t)) {
                result.add(t);
            }
        }
        return result;
    }

    protected abstract boolean accept(T t);

}
