package se.su.dsv.scipro.finalseminar;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.CompoundPropertyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.ResourceModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import org.springframework.data.domain.PageRequest;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.components.OppositeVisibility;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.List;
import java.util.stream.Collectors;

public class ProjectActiveParticipationListPanel extends GenericPanel<Project> {
    @Inject
    private FinalSeminarService finalSeminarService;

    public ProjectActiveParticipationListPanel(final String id, final IModel<Project> model) {
        super(id, model);
        add(new AutoHidingListView<FinalSeminarActiveParticipation>("active.participations", getActiveParticipants(getSeminars())) {
            @Override
            protected void populateItem(final ListItem<FinalSeminarActiveParticipation> item) {
                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(Project.class), item.getModelObject().getFinalSeminar().getProject().getId());

                BookmarkablePageLink<Void> finalSeminarLink = new BookmarkablePageLink<>("finalSeminar", ProjectFinalSeminarDetailsPage.class, pp);
                finalSeminarLink.add(new DateLabel("finalSeminar.startDate", DateStyle.DATE));
                item.add(finalSeminarLink);
                item.add(new UserLinkPanel("user"));
            }

            @Override
            protected IModel<FinalSeminarActiveParticipation> getListItemModel(final IModel<? extends List<FinalSeminarActiveParticipation>> listViewModel, final int index) {
                return new CompoundPropertyModel<>(super.getListItemModel(listViewModel, index));
            }
        });

        add(new Label("no.active.participation", new ResourceModel("no.active.participation"))
                .add(new OppositeVisibility(get("active.participations"))));
    }

    private IModel<? extends List<FinalSeminarActiveParticipation>> getActiveParticipants(final IModel<List<FinalSeminar>> seminars) {
        return new LoadableDetachableModel<List<FinalSeminarActiveParticipation>>() {
            @Override
            protected List<FinalSeminarActiveParticipation> load() {
                return seminars.getObject().stream()
                        .flatMap(input -> input.getActiveParticipations().stream())
                        .filter(input -> getModelObject().isParticipant(input.getUser()))
                        .collect(Collectors.toList());
            }

            @Override
            protected void onDetach() {
                seminars.detach();
            }
        };
    }

    private IModel<List<FinalSeminar>> getSeminars() {
        return new LoadableDetachableModel<List<FinalSeminar>>() {
            @Override
            protected List<FinalSeminar> load() {
                return finalSeminarService.findProjectParticipating(getModelObject(), new PageRequest(0, Integer.MAX_VALUE));
            }
        };
    }
}