package se.su.dsv.scipro.grading;

import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.Calendar;

public class TemplatePanel extends Panel {

    public static final String BACHELOR_CONTAINER = "bachelorContainer";
    public static final String MASTER_15_CONTAINER = "master15Container";
    public static final String MASTER_30_CONTAINER = "master30Container";

    public TemplatePanel(String id, final IModel<Project> projectIModel) {
        super(id, projectIModel);
        final DegreeLevel degreeLevel = projectIModel.getObject().getProjectType().getDegreeLevel();
        final int credits = projectIModel.getObject().getCredits();

        final WebMarkupContainer upto16 = new WebMarkupContainer("upto2016") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getYear(projectIModel.getObject()) < 2017);
            }
        };
        addTemplates(degreeLevel, credits, upto16);
        add(upto16);
        final WebMarkupContainer from2017 = new WebMarkupContainer("from2017") {
            @Override
            protected void onConfigure() {
                setVisibilityAllowed(getYear(projectIModel.getObject()) >= 2017);
            }
        };
        addTemplates(degreeLevel, credits, from2017);
        add(from2017);
    }

    private void addTemplates(final DegreeLevel degreeLevel, final int credits, final WebMarkupContainer container) {
        container.add(new WebMarkupContainer(BACHELOR_CONTAINER) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(degreeLevel.equals(DegreeLevel.FIRST_CYCLE));
            }
        });
        container.add(new WebMarkupContainer(MASTER_15_CONTAINER) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(degreeLevel.equals(DegreeLevel.SECOND_CYCLE) && credits == ProjectType.HP_15);
            }
        });
        container.add(new WebMarkupContainer(MASTER_30_CONTAINER) {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(degreeLevel.equals(DegreeLevel.SECOND_CYCLE) && (credits == ProjectType.HP_30 || credits == 0));
            }
        });
    }

    private int getYear(final Project project) {
        final Calendar calendar = Calendar.getInstance();
        calendar.setTime(project.getDateCreated());
        return calendar.get(Calendar.YEAR);
    }

}
