package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.link.ResourceLink;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.resource.CharSequenceResource;

import java.util.stream.Collectors;

public class IdeaDownloadPanel extends GenericPanel<Idea> {

    private final Idea idea;

    public IdeaDownloadPanel(String id, IModel<Idea> model) {
        super(id, model);
        idea = model.getObject();
        String ideaString = getIdeaString();
        CharSequenceResource resource = new CharSequenceResource("text/plain; charset=utf8", ideaString, idea.getTitle() + ".txt");
        add(new ResourceLink<Idea>("download", resource));
    }

    private String getIdeaString() {
        StringBuilder sb = new StringBuilder("<<IDEA SPECIFICATION>>");
        String separator = System.getProperty("line.separator");
        sb.append(separator).append(separator);
        sb.append("Title: ").append(separator).append(idea.getTitle()).append(separator).append(separator);
        sb.append("Description: ").append(separator).append(idea.getDescription()).append(separator).append(separator);
        sb.append("Type: ").append(separator).append(idea.getType()).append(separator).append(separator);
        sb.append("Language: ").append(separator).append(getLanguageNames(idea)).append(separator).append(separator);
        sb.append("Prerequisites: ").append(separator).append(idea.getPrerequisites()).append(separator).append(separator);
        if (idea.getResearchArea() != null) {
            sb.append("ResearchArea: ").append(separator).append(idea.getResearchArea().getTitle()).append(separator).append(separator);
        }
        sb.append("Keywords: ").append(separator).append(idea.getKeywordNames()).append(separator).append(separator);
        sb.append("Authors: ").append(separator).append(idea.getAuthorNames()).append(separator).append(separator);
        sb.append("Application period: ").append(separator).append(idea.getApplicationPeriod() == null ? "" : idea.getApplicationPeriod().getName()).append(separator).append(separator);
        sb.append("<<\"Student idea form\">>").append(separator).append(separator);
        sb.append("Background? ").append(separator).append(idea.getTholanderBox().getBackground()).append(separator).append(separator);
        sb.append("Literature? ").append(separator).append(idea.getTholanderBox().getLiterature()).append(separator).append(separator);
        sb.append("Problem? ").append(separator).append(idea.getTholanderBox().getProblem()).append(separator).append(separator);
        sb.append("Method? ").append(separator).append(idea.getTholanderBox().getMethod()).append(separator).append(separator);
        sb.append("Interests? ").append(separator).append(idea.getTholanderBox().getInterests()).append(separator);
        return sb.toString();
    }

    private String getLanguageNames(final Idea idea) {
        return idea.getLanguages()
                .stream()
                .map(language -> getString("Language." + language.name()))
                .collect(Collectors.joining(", "));
    }
}
