package se.su.dsv.scipro.match;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.applicationperiod.ApplicationPeriodInfoPanel;
import se.su.dsv.scipro.applicationperiod.ApplicationPeriodSupervisorPanel;
import se.su.dsv.scipro.components.AutoHidingListView;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;
import java.util.List;

public class SupervisorApplicationPeriodInfoPanel extends Panel {
    @Inject
    private ApplicationPeriodService periodService;
    @Inject
    private TargetService targetService;

    static final String PERIOD_LIST = "currentPeriods";
    static final String EMPTY = "emptyLabel";
    static final String SELECT_STUDENT = "selectStudentIdeas";

    public SupervisorApplicationPeriodInfoPanel(String id) {
        super(id);

        final IModel<List<ApplicationPeriod>> currentPeriods = new LoadableDetachableModel<List<ApplicationPeriod>>() {
            @Override
            protected List<ApplicationPeriod> load() {
                return periodService.getCurrentSupervisorPeriods(SciProSession.get().getUser());
            }
        };

        add(new AutoHidingListView<ApplicationPeriod>(PERIOD_LIST, currentPeriods) {
            @Override
            protected void populateItem(final ListItem<ApplicationPeriod> item) {
                item.add(new ApplicationPeriodInfoPanel("info", item.getModel()));

                PageParameters pp = new PageParameters();
                pp.add(PageParameterKeys.MAP.get(ApplicationPeriod.class), item.getModelObject().getId());

                item.add(new BookmarkablePageLink<Void>(SELECT_STUDENT, SupervisorSelectableStudentIdeasPage.class, pp) {
                    @Override
                    protected void onConfigure() {
                        super.onConfigure();
                        setVisibilityAllowed(targetService.hasTargetsLeft(item.getModelObject(), SciProSession.get().getUser()));
                    }
                });

                item.add(new ApplicationPeriodSupervisorPanel("sup", item.getModel()));
            }
        });

        add(new Label(EMPTY, "No application period open") {
            @Override
            protected void onConfigure() {
                super.onConfigure();
                setVisibilityAllowed(currentPeriods.getObject().isEmpty());
            }
        });
    }
}
