package se.su.dsv.scipro.oauth;

import org.apache.wicket.markup.html.WebPage;
import org.apache.wicket.request.flow.RedirectToUrlException;
import org.apache.wicket.request.mapper.parameter.PageParameters;

import javax.inject.Inject;
import java.time.Instant;
import java.util.Objects;

public class CallbackPage extends WebPage {
    @Inject
    private OAuthService oAuthService;

    public CallbackPage(PageParameters pageParameters) {
        final String code = pageParameters.get("code").toOptionalString();
        final String state = pageParameters.get("state").toOptionalString();
        final String returnUrl = getSession().getMetaData(OAuth.RETURN_URL);
        final String expectedState = getSession().getMetaData(OAuth.STATE);

        if (Objects.equals(expectedState, state)) {
            final TokenResponse tokenResponse = oAuthService.exchangeForToken(code);
            if (tokenResponse != null) {
                getSession().setMetaData(OAuth.TOKEN, tokenResponse.getAccessToken());
                getSession().setMetaData(OAuth.EXPIRATION, Instant.now().plusSeconds(tokenResponse.getExpiresIn()));
                if (returnUrl != null) {
                    throw new RedirectToUrlException(returnUrl);
                }
                else {
                    setResponsePage(getApplication().getHomePage());
                }
            }
            else {
                getSession().error("Authorization failed");
                setResponsePage(getApplication().getHomePage());
            }
        }
        else {
            getSession().error("Authorization failed");
            setResponsePage(getApplication().getHomePage());
        }
    }
}
