package se.su.dsv.scipro.profile;

import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.WebComponent;
import org.apache.wicket.model.IModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

public class UserProfileImage extends WebComponent {
    public enum Size {
        TINY(16), MEDIUM(64), LARGE(128);

        private final int pixels;

        Size(final int pixels) {
            this.pixels = pixels;
        }

        int getPixels() {
            return pixels;
        }
    }

    private final IModel<User> user;
    private final Size size;

    public UserProfileImage(final String id, final IModel<User> user, final Size size) {
        super(id, user);
        this.user = user;
        this.size = size;
    }

    @Override
    protected void onComponentTag(final ComponentTag tag) {
        checkComponentTag(tag, "img");
        super.onComponentTag(tag);
        tag.put("src", getImageSrc());
        tag.put("style", String.format("max-width: %dpx; max-height: %dpx;", size.getPixels(), size.getPixels()));
        tag.put("alt", user.getObject().getFullName());
    }

    private CharSequence getImageSrc() {
        final PageParameters pp = new PageParameters();
        pp.set(DaisyPicture.USER_ID, user.getObject().getIdentifier());
        pp.set("x", SciProSession.get().getUser().getId());
        return urlFor(new DaisyPicture(), pp);
    }
}
