package se.su.dsv.scipro.statistics;

import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.components.EmployeeAutoComplete;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;

import javax.inject.Inject;

public class SupervisorStatisticsPanel extends Panel {

	@Inject
	private UserService userService;

	private final SupervisorApplicationPeriodStatisticsPanel periodStatisticsPanel;
	private SupervisorProjectStatisticsPanel projectPanel;
	private SupervisorIdeaStatisticsPanel ideaPanel;

	public SupervisorStatisticsPanel(String id) {
		super(id);
		addEmployeeSelection();
        projectPanel = new SupervisorProjectStatisticsPanel(PROJECTS_PANEL);
		projectPanel.setOutputMarkupPlaceholderTag(true);
		add(projectPanel);
        ideaPanel = new SupervisorIdeaStatisticsPanel(IDEAS_PANEL);
		ideaPanel.setOutputMarkupPlaceholderTag(true);
		add(ideaPanel);
		periodStatisticsPanel = new SupervisorApplicationPeriodStatisticsPanel("periods", new Model<User>()) {
			@Override
			protected void onConfigure() {
				setVisible(getModelObject() != null);
			}
		};
		periodStatisticsPanel.setOutputMarkupPlaceholderTag(true);
		add(periodStatisticsPanel);
	}

	private void addEmployeeSelection() {
		add(new EmployeeAutoComplete(SUPERVISOR_SELECTION, new DetachableServiceModel<>(userService)) {
			

			@Override
			protected void action(AjaxRequestTarget pTarget, User newSelection) {
				projectPanel.updateSupervisor(newSelection);
				ideaPanel.updateSupervisor(newSelection);
				periodStatisticsPanel.setModel(getModel());
                pTarget.add(projectPanel);
                pTarget.add(ideaPanel);
				pTarget.add(periodStatisticsPanel);
			}
		});
	}
	
	static final String SUPERVISOR_SELECTION = "employeeSelection";
	static final String PROJECTS_PANEL = "projectPanel";
	static final String IDEAS_PANEL = "ideaPanel";
}
