package se.su.dsv.scipro.supervisor.pages;

import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.RequiredTextField;
import org.apache.wicket.markup.html.form.TextArea;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.BootstrapDateTimeComponent;
import se.su.dsv.scipro.firstmeeting.FirstMeetingService;
import se.su.dsv.scipro.project.Project;

import javax.inject.Inject;
import java.util.Date;

public class SupervisorFirstMeetingPage extends AbstractSupervisorProjectDetailsPage {
    @Inject
    private FirstMeetingService firstMeetingService;

    private IModel<Date> date = new Model<>();
    private IModel<String> room = new Model<>();
    private IModel<String> description = new Model<>();

    public SupervisorFirstMeetingPage(final PageParameters pp) {
        super(pp);

        firstMeetingService.findByProject(projectModel.getObject()).ifPresent(firstMeeting -> {
            date.setObject(firstMeeting.getDate());
            room.setObject(firstMeeting.getRoom());
            description.setObject(firstMeeting.getDescription());
        });

        add(new FeedbackPanel("feedback"));

        add(new Form<Project>("form", projectModel) {
            {
                add(new BootstrapDateTimeComponent("date", date).setRequired(true));
                add(new RequiredTextField<>("room", room));
                add(new TextArea<>("description", description));
            }

            @Override
            protected void onSubmit() {
                success("First meeting scheduled");
                firstMeetingService.schedule(projectModel.getObject(), date.getObject(), room.getObject(), description.getObject());
            }
        });
    }
}
