package se.su.dsv.scipro.supervisor.panels;

import com.google.inject.Inject;
import org.apache.wicket.feedback.FencedFeedbackPanel;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.form.upload.FileUploadField;
import org.apache.wicket.markup.html.panel.GenericPanel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.util.ListModel;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.files.WicketFileUpload;
import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.reviewing.ReviewerApproval;
import se.su.dsv.scipro.reviewing.ReviewerApprovalService;
import se.su.dsv.scipro.system.Language;
import se.su.dsv.scipro.system.ResearchArea;
import se.su.dsv.scipro.system.ResearchAreaService;

import java.util.Arrays;
import java.util.List;

public abstract class SupervisorApprovalPanel<T extends ReviewerApproval> extends GenericPanel<Project> {

    @Inject
    private ResearchAreaService researchAreaService;

    public SupervisorApprovalPanel(String id, IModel<Project> model) {
        super(id, model);

        add(new RequestFinalSeminarApprovalForm("requestApproval", model));
        add(new FinalSeminarApprovalProcessPanel("process", getProcess(model)) {
            @Override
            protected void onConfigure() {
                setVisible(getModelObject() != null);
            }
        });
    }

    protected IModel<ReviewerApproval> getProcess(final IModel<Project> project) {
        return new LoadableDetachableModel<ReviewerApproval>() {
            @Override
            protected ReviewerApproval load() {
                return getService().findBy(project.getObject()).getOrElse(() -> null);
            }
        };
    }

    protected abstract ReviewerApprovalService<T> getService();

    protected class RequestFinalSeminarApprovalForm extends StatelessForm<Project> {
        private final FileUploadField thesisUpload;
        private IModel<String> comment = new Model<>();

        public RequestFinalSeminarApprovalForm(final String id, final IModel<Project> project) {
            super(id, project);

            add(new FencedFeedbackPanel("feedback", this));
            add(new TextArea<>("comment", comment));

            thesisUpload = new FileUploadField("thesis");
            thesisUpload.setRequired(true);
            add(thesisUpload);

            List<ResearchArea> areas = researchAreaService.findNotDeleted();

            DropDownChoice<ResearchArea> researchArea = new DropDownChoice<>("area", new PropertyModel<ResearchArea>(getModel(), "researchArea"),
                    new ListModel<>(areas), new ChoiceRenderer<ResearchArea>("title", "id"));
            add(researchArea);

            final DropDownChoice<Language> language =
                    new DropDownChoice<>(
                            "language",
                            new PropertyModel<>(getModel(), "language"),
                            Arrays.asList(Language.values()),
                            new EnumChoiceRenderer<>(this));
            language.setRequired(true);
            add(language);
        }

        @Override
        protected void onConfigure() {
            setVisible(getService().requiresUpload(getModelObject()));
        }

        @Override
        protected void onSubmit() {
            Project project = getModelObject();
            FileUpload fileUpload = WicketFileUpload.from(thesisUpload.getFileUpload());
            getService().requestApproval(project, fileUpload, comment.getObject());
        }
    }
}
