package se.su.dsv.scipro.applicationperiod;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectType;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;

import static se.su.dsv.scipro.applicationperiod.ApplicationPeriodInfoPanel.*;


@RunWith(MockitoJUnitRunner.class)
public class ApplicationPeriodInfoPanelTest extends SciProTest {

    ApplicationPeriodInfoPanel panel;
    private ApplicationPeriod bachelor;

    @Before
    public void setUp() throws Exception {

        bachelor = new ApplicationPeriod("Bachelor");
        bachelor.setStartDate(new Date(0));
        bachelor.setEndDate(new Date(86400));
        bachelor.setCourseStartDate(new Date(86400 * 2));
        bachelor.setId(1L);
        ProjectType pc = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        bachelor.setProjectTypes(Collections.singleton(pc));
        panel = tester.startComponentInPage(new ApplicationPeriodInfoPanel("panel", Model.of(bachelor)));
    }

    @Test
    public void testRenders() throws Exception {
        tester.assertNoErrorMessage();
    }

    @Test
    public void testContainsListOfProjectTypes() throws Exception {
        tester.assertComponent(path(panel, PROJECT_TYPES), ListView.class);
        tester.assertModelValue(path(panel, PROJECT_TYPES), new ArrayList<>(bachelor.getProjectTypes()));
    }

    @Test
    public void testLabelForEachPeriod() throws Exception {
        tester.assertComponent(path(panel, NAME), Label.class);
        tester.assertModelValue(path(panel, NAME), bachelor.getName());
    }

    @Test
    public void testStartDateLabelForEachPeriod() throws Exception {
        tester.assertComponent(path(panel, START_DATE), DateLabel.class);
        tester.assertModelValue(path(panel, START_DATE), bachelor.getStartDate());
    }

    @Test
    public void testEndDateLabelForEachPeriod() throws Exception {
        tester.assertComponent(path(panel, END_DATE), DateLabel.class);
        tester.assertModelValue(path(panel, END_DATE), bachelor.getEndDate());
    }

    @Test
    public void testCourseStartLabelForEachPeriod() throws Exception {
        tester.assertComponent(path(panel, COURSE_START_DATE), DateLabel.class);
        tester.assertModelValue(path(panel, COURSE_START_DATE), bachelor.getCourseStartDate());
    }
}
