package se.su.dsv.scipro.match;

import org.junit.Test;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.system.User;

import java.util.Collections;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.when;
import static se.su.dsv.scipro.match.StudentApplicationPeriodInfoPanel.*;

public class StudentApplicationPeriodInfoPanelTest extends SciProTest {
    @Test
    public void can_create_ideas_in_exempted_periods() throws Exception {
        final ApplicationPeriod somePeriod = createSomePeriod();
        when(applicationPeriodService.getCurrentStudentPeriods(any(User.class))).thenReturn(Collections.singletonList(new ExemptedPeriod(somePeriod)));
        final StudentApplicationPeriodInfoPanel panel = tester.startComponentInPage(StudentApplicationPeriodInfoPanel.class);
        tester.assertVisible(path(panel, PERIOD_LIST, 0, STUDENT_CONTAINER, CREATE));
    }

    @Test
    public void can_create_ideas_in_open_periods() throws Exception {
        final ApplicationPeriod somePeriod = createSomePeriod();
        when(applicationPeriodService.getCurrentStudentPeriods(any(User.class))).thenReturn(Collections.singletonList(new OpenPeriod(somePeriod)));
        final StudentApplicationPeriodInfoPanel panel = tester.startComponentInPage(StudentApplicationPeriodInfoPanel.class);
        tester.assertVisible(path(panel, PERIOD_LIST, 0, STUDENT_CONTAINER, CREATE));
    }

    @Test
    public void can_not_create_or_select_idea_if_already_participant() throws Exception {
        final ApplicationPeriod somePeriod = createSomePeriod();
        when(applicationPeriodService.getCurrentStudentPeriods(any(User.class))).thenReturn(Collections.singletonList(new OpenPeriod(somePeriod)));
        when(ideaService.authorParticipatingOnActiveIdea(any(User.class), eq(somePeriod))).thenReturn(true);
        final StudentApplicationPeriodInfoPanel panel = tester.startComponentInPage(StudentApplicationPeriodInfoPanel.class);
        tester.assertInvisible(path(panel, PERIOD_LIST, 0, STUDENT_CONTAINER));
    }

    private ApplicationPeriod createSomePeriod() {
        final ApplicationPeriod applicationPeriod = new ApplicationPeriod();
        applicationPeriod.setId(1L);
        return applicationPeriod;
    }
}