package se.su.dsv.scipro.projectpartner;

import org.apache.wicket.util.tester.FormTester;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Matchers;
import org.mockito.Mockito;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettings;
import se.su.dsv.scipro.match.ApplicationPeriod;
import se.su.dsv.scipro.system.DegreeLevel;
import se.su.dsv.scipro.system.ProjectModule;
import se.su.dsv.scipro.system.ProjectType;
import se.su.dsv.scipro.test.ObjectMother;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import static org.mockito.Matchers.anyLong;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.when;

public class ProjectPartnerPageTest extends SciProTest {

    private ApplicationPeriod applicationPeriod;

    @Before
    public void setUp() throws Exception {
        GeneralSystemSettings settings = new GeneralSystemSettings();
        int daysToLive = 3;
        final ProjectType bachelor = new ProjectType(DegreeLevel.FIRST_CYCLE, "Bachelor", "Bachelor");
        bachelor.setId(9374L);
        final ProjectType master = new ProjectType(DegreeLevel.SECOND_CYCLE, "Master", "Master");
        master.setId(2409875L);
        final List<ProjectType> projectTypes = Arrays.asList(bachelor, master);
        when(projectTypeService.findWithModule(ProjectModule.MATCH)).thenReturn(projectTypes);
        applicationPeriod = new ApplicationPeriod("Test period");
        applicationPeriod.setCourseStartDate(new Date(new Date().getTime() + 86400*1000));
        applicationPeriod.addProjectType(bachelor);
        applicationPeriod.addProjectType(master);
        applicationPeriod.setId(234L);
        when(applicationPeriodService.findAll()).thenReturn(Collections.singletonList(applicationPeriod));
        when(applicationPeriodService.findOne(anyLong())).thenReturn(applicationPeriod);
        settings.setProjectPartnerDaysToLive(daysToLive);
        when(generalSystemSettingsService.getGeneralSystemSettingsInstance()).thenReturn(settings);
        when(projectPartnerService.getProjectPartnerInSpan(eq(daysToLive), eq(applicationPeriod), Matchers.any(ProjectType.class), Matchers.any(Pageable.class))).thenReturn(Collections.singletonList(new ProjectPartner(ObjectMother.SOME_USER)));
        when(projectPartnerService.countProjectPartnerInSpan(eq(daysToLive), eq(applicationPeriod), Matchers.any(ProjectType.class))).thenReturn(1L);
        tester.startPage(ProjectPartnerPage.class);
    }

    @Test
    public void renders() throws Exception {
        tester.assertRenderedPage(ProjectPartnerPage.class);
    }

    @Test
    public void submit_new_ad() throws Exception {
        tester.getRequest().getPostParameters().setParameterValue("applicationPeriod", Long.toString(applicationPeriod.getId()));
        tester.executeAjaxEvent("applicationPeriod", "change");
        FormTester formTester = tester.newFormTester(path("panels", "projectPartnerForm"));
        formTester.setValue("infotext", "help!");
        formTester.select("projectType", 0);
        formTester.submit();

        Mockito.verify(projectPartnerService).save(Matchers.any(ProjectPartner.class));
    }
}
