package se.su.dsv.scipro.repository.panels;

import org.apache.wicket.model.Model;
import org.junit.Before;
import org.junit.Test;
import org.springframework.data.domain.Pageable;
import se.su.dsv.scipro.SciProTest;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.ProjectFile;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.test.DomainObjects;
import se.su.dsv.scipro.test.ObjectMother;
import se.su.dsv.scipro.test.UserBuilder;

import java.util.Collections;
import java.util.Date;

import static org.mockito.Matchers.any;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class ProjectFilePanelTest extends SciProTest {

    public static final String DELETE_LINK_PATH = "id:table:body:rows:1:cells:6:cell:link";
    private ProjectFile projectFile;
    private ProjectFilePanel panel;

    @Before
    public void setUp() throws Exception {
        User uploader = new UserBuilder().create();
        setLoggedInAs(uploader);
        projectFile = new ProjectFile();
        projectFile.setProject(ObjectMother.SOME_PROJECT);
        final FileDescription fileDescription = new FileDescription();
        fileDescription.setUploader(uploader);
        fileDescription.setSize(100L);
        fileDescription.setName("My thesis.pdf");
        fileDescription.setMimeType("application/pdf");
        projectFile.setDateCreated(new Date());
        projectFile.setFileDescription(fileDescription);
        DomainObjects.injectId(projectFile, 1L);
        DomainObjects.injectId(fileDescription, 1L);

        when(projectFileService.findOne(projectFile.getId())).thenReturn(projectFile);
        when(projectFileService.getProjectFiles(eq(ObjectMother.SOME_PROJECT), any(Pageable.class))).thenReturn(Collections.singletonList(projectFile));
        when(projectFileService.countProjectFiles(ObjectMother.SOME_PROJECT)).thenReturn(1L);
    }

    @Test
    public void renders() throws Exception {
        startPanel();
    }

    @Test
    public void delete_link_visible_for_deletable_files() throws Exception {
        startPanel();
        tester.assertVisible(DELETE_LINK_PATH);
    }

    @Test
    public void delete_link_not_visible_if_not_uploader() throws Exception {
        setLoggedInAs(new UserBuilder().create());
        startPanel();
        tester.assertInvisible(DELETE_LINK_PATH);
    }

    @Test
    public void click_delete_calls_service() throws Exception {
        startPanel();
        tester.clickLink(DELETE_LINK_PATH);
        verify(projectFileService).delete(projectFile);
    }

    private void startPanel() {
        panel = tester.startComponentInPage(new ProjectFilePanel("id", Model.of(ObjectMother.SOME_PROJECT)));
    }

    @Override
    protected String path(Object... parts) {
        return super.path(panel, super.path(parts));
    }
}
